﻿using System;
using System.IO;

using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;

using AutoCAD = Autodesk.AutoCAD.ApplicationServices.Application;

namespace AcDgLibrary
{
    public class AcDgBlock
    {
        private Document _dwg;
        private Database _database;
        private Transaction _transaction;
        
        private string _filename = string.Empty;
        private string _blockname = string.Empty;

        private double _rotation = 0;
        private Point3d _insertionpoint = new Point3d(0, 0, 0);
        private Scale3d _scalefactors = new Scale3d(1);
        private ObjectId _objectid = ObjectId.Null;

        public AcDgBlock(Document dwg, Transaction trans)
        {
            _dwg = dwg;
            _database = _dwg.Database;
            _transaction = trans;
        }

        public string BlockName
        {
            set { _blockname = value; }
        }

        public string DwgFileName
        {
            set { _filename = value; }
        }
        
        public Point3d InsertionPoint
        {
            set { _point = value; }
        }

        public double Rotation
        {
            set { _rotation = value; }
        }

        public Scale3d ScaleFactors
        {
            set { _scalefactors = value; }
        }

        public bool HasBlock
        {
            get 
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    BlockTable bt = (BlockTable)_transaction.GetObject(_database.BlockTableId, OpenMode.ForRead);
                    if (bt.Has(_blockname)) return true;
                    else return false;
                }
            }
        }

        public ObjectId AddBlockToBlockTable()
        {
            try
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    BlockTable bt = (BlockTable)_transaction.GetObject(_database.BlockTableId, OpenMode.ForRead);
                    if (bt.Has(Path.GetFileNameWithoutExtension(_filename)))
                    {
                        return _objectid = bt[Path.GetFileNameWithoutExtension(_filename)];
                    }
                    else
                    {
                        using (Database dbDwg = new Database(false, true))
                        {
                            if (File.Exists(_filename))
                            {
                                dbDwg.ReadDwgFile(_filename, FileShare.Read, true, "");
                                return _objectid = _database.Insert(Path.GetFileNameWithoutExtension(_filename), dbDwg, true);
                            }
                            else return _objectid;
                        }
                    }
                }
            }
            catch(SystemException ex)
            {
                AutoCAD.ShowAlertDialog(ex.Message);
                return _objectid;
            }
        }

        public BlockTableRecord GetBlockTableRecord
        {
            get
            {
                BlockTableRecord btr = null;

                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    BlockTable bt = (BlockTable)_transaction.GetObject(_database.BlockTableId, OpenMode.ForRead);
                    if (bt.Has(_blockname))
                    {
                        btr = (BlockTableRecord)_transaction.GetObject(bt[_blockname], OpenMode.ForWrite);
                        return btr;
                    }
                    else
                    {
                        return btr;
                    }
                }
            }
        }

        public void CreateNewBlockReference()
        {
            try
            {
                using (DocumentLock dwgLck = _dwg.LockDocument())
                {
                    BlockTable bt = (BlockTable)_transaction.GetObject(_database.BlockTableId, OpenMode.ForRead);
                    if (!bt.Has(_blockname)) return;
                    
                    BlockTableRecord btr = (BlockTableRecord)_transaction.GetObject(_database.CurrentSpaceId, OpenMode.ForWrite);
                    using (BlockReference blkRef = new BlockReference(_insertionpoint, bt[_blockname]))
                    {
                        blkRef.ScaleFactors = _scalefactors;
                        blkRef.Rotation = _rotation;
                        btr.AppendEntity(blkRef);
                        _transaction.AddNewlyCreatedDBObject(blkRef, true);
                    }
                }
            }
            catch (SystemException ex)
            {
                AutoCAD.ShowAlertDialog(ex.Message);
            }
        }
    }
}
