﻿using System;

using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.DatabaseServices;

namespace AcDgLibrary
{
    public class AcDgLayer
    {
        private Document _dwg;
        private Database _database;
        private Transaction _transaction;
        
        private AcDgEnums.LineTypes _linetype = AcDgEnums.LineTypes.CONTINUOUS;
        private Color _color = Color.FromColorIndex(ColorMethod.ByAci, 7);
        private bool _isfrozen = false;
        private bool _ishidden = false;
        private bool _islocked = false;
        private bool _isoff = false;
        private bool _isplottable = true;
        private bool _isused = false;
        private ObjectId _objectid = ObjectId.Null;
        private string _name = "0";

        //Constructor
        public AcDgLayer(Document dwg, Transaction trans)
        {
            _dwg = dwg;
            _database = _dwg.Database;
            _transaction = trans;
        }

        public string Name
        {
            set { _name = value; }
        }

        public Color Color
        {
            set { _color = value; }
        }

        public AcDgEnums.LineTypes Linetype
        {
            set { _linetype = value; }
        }

        public bool IsFrozen
        {
            set { _isfrozen = value; }
        }

        public bool IsHidden
        {
            set { _ishidden = value; }
        }

        public bool IsLocked
        {
            set { _islocked = value; }
        }

        public bool IsOff
        {
            set { _isoff = value; }
        }

        public bool IsPlottable
        {
            set { _isplottable = value; }
        }

        public bool HasLayer
        {
            get
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    LayerTable lt = (LayerTable)_transaction.GetObject(_database.LayerTableId, OpenMode.ForRead);
                    if (lt.Has(_name)) return true;
                    else return false;
                }
            }
        }

        public bool IsUsed
        {
            get 
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    ObjectId ltId = _database.LayerTableId;
                    LayerTable lt = (LayerTable)_transaction.GetObject(ltId, OpenMode.ForRead);
                    if (lt.Has(_name))
                    {
                        ObjectId ltrId = lt[_name];

                        ObjectIdCollection idCol = new ObjectIdCollection();
                        idCol.Add(ltrId);
                        _database.Purge(idCol);

                        if (idCol.Count != 0)
                        {
                            _isused = false;
                        }
                        else
                        {
                            _isused = true;
                        }
                    }
                    else
                    {
                       _isused = false;
                    }
                }
                return _isused;
            }
        }

        public LayerTableRecord GetLayerTableRecord
        {
            get
            {
                LayerTableRecord ltr = null;

                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    LayerTable lt = (LayerTable)_transaction.GetObject(_database.LayerTableId, OpenMode.ForWrite);
                    if (lt.Has(_name))
                    {
                        ltr = (LayerTableRecord)_transaction.GetObject(lt[_name], OpenMode.ForWrite);
                        return ltr;
                    }
                    else
                    {
                        return ltr;
                    }
                }
            }
        }

        public ObjectId Create()
        {
            using (DocumentLock dwglck = _dwg.LockDocument())
            {
                LayerTable lt = (LayerTable)_transaction.GetObject(_database.LayerTableId, OpenMode.ForWrite);
                if (lt.Has(_name))
                {
                    _objectid = lt[_name];
                }
                else
                {
                    LayerTableRecord ltr = new LayerTableRecord();
                    ltr.Name = _name;
                    ltr.Color = _color;
                    ltr.IsFrozen = _isfrozen;
                    ltr.IsHidden = _ishidden;
                    ltr.IsLocked = _islocked;
                    ltr.IsOff = _isoff;
                    ltr.IsPlottable = _isplottable;
                    ltr.LinetypeObjectId = new AcDgLineType(_dwg, _transaction, _linetype).GetLineTypeObjectId;
                    
                    _objectid = lt.Add(ltr);
                    _transaction.AddNewlyCreatedDBObject(ltr, true);
                }
                return _objectid;
            }
        }
    }
}
